﻿'_____________________________________

'L E G O   N X T   R E M O T E   C O N T R O L
'                      Ron Kessler
'                     10/27/2011
'_____________________________________

'Purpose: This controls a Lego NXT via Bluetooth
'Be sure to pair the NXT with your computer first and note the Com port #

'When you run the app enter the com port # and click connect. Drive your bot using the buttons or arrow keys.
'The end key stops it.
'Be sure to disconnect when you finish.

'Form properties: KeyPreview = True

'Updated 10/30/2011

Option Explicit On
Option Strict On

Imports NKH.MindSqualls
Imports NXT_BT_Demo.General


Public Class frmMain

#Region "Constructor & Form Activated"
    Public Sub New()
        InitializeComponent()
        grpMotors.Enabled = False
        lblStatus.Text = DisconnectedMsg

        '---set power to 25% to start
        scrPower.Value = 25
        PowerLevel = 25


    End Sub

    Private Sub frmMain_Activated(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Activated
        txtPort.SelectAll()
    End Sub
#End Region


#Region "Make BT Connection"
    Private Sub btnConnect_Click(sender As System.Object, e As System.EventArgs) Handles btnConnect.Click
        Try
            Dim comPort As Byte = Byte.Parse(txtPort.Text.Trim) 'get the com port

            '---create new brick and tell it to use BlueTooth
            myBrick = New NxtBrick(NxtCommLinkType.Bluetooth, comPort)

            '---create two motors
            myBrick.MotorB = New NxtMotor
            myBrick.MotorC = New NxtMotor

            '---this creates a synch object that will drive motors straight
            motorPair = New NxtMotorSync(myBrick.MotorB, myBrick.MotorC)

            '---now connect
            myBrick.Connect()
            If myBrick.IsConnected Then
                lblStatus.Text = ConnectedMsg
                grpMotors.Enabled = True
                myBattery = CStr(FormatNumber(myBrick.BatteryLevel / 1000, 1))
                Me.lblBattery.Text = myBattery.ToString & " volts"
            Else
                lblStatus.Text = DisconnectedMsg
                grpMotors.Enabled = False
            End If

            txtPort.Focus()         'keeps buttons from handling my arrow keydown events
        Catch ex As Exception
            MessageBox.Show("Unable to connect." & ex.Message, "System Message", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Sub
#End Region

#Region "Power level"
    Private Sub scrPower_ValueChanged_1(sender As System.Object, e As System.EventArgs) Handles scrPower.ValueChanged
        PowerLevel = CByte(scrPower.Value)
        If Not LastButtonClicked Is Nothing Then
            LastButtonClicked.PerformClick()
      
        End If
    End Sub
#End Region

#Region "Direction buttons and Arrow Keys"
    Private Sub Direction_Controls(sender As System.Object, e As System.EventArgs) Handles btnForward.Click,
        btnRight.Click, btnReverse.Click, btnLeft.Click

        Dim whichButton As Button = CType(sender, Button)

        '---hold name of last button
        LastButtonClicked = whichButton

        Select Case whichButton.Name
            Case "btnForward"
                GoForward()
            Case "btnReverse"
                GoBackwards()
            Case "btnRight"
                TurnRight()
            Case "btnLeft"
                TurnLeft()
        End Select
        txtPort.Focus()         'keeps buttons from handling my arrow keydown events
    End Sub


    Private Sub frmMain_KeyDown(sender As Object, e As System.Windows.Forms.KeyEventArgs) Handles Me.KeyDown
        '---make sure form.keypreview = true
        Select Case e.KeyCode
            Case Keys.Left
                e.Handled = True        'don't pass the key to any control that might have the focus
                btnLeft.PerformClick()
            Case Keys.Right
                e.Handled = True
                btnRight.PerformClick()
            Case Keys.Up
                e.Handled = True
                btnForward.PerformClick()
            Case Keys.Down
                e.Handled = True
                btnReverse.PerformClick()
            Case Keys.End
                e.Handled = True
                btnIdle.PerformClick()
        End Select
        txtPort.Focus()         'keeps buttons from handling my arrow keydown events
    End Sub

#End Region

#Region "Move Bot"
    Private Sub GoForward()
        Idle()
        motorPair.Run(CSByte(PowerLevel), 0, 0)
    End Sub

    Private Sub GoBackwards()
        Idle()
        motorPair.Run(CSByte(-PowerLevel), 0, 0)
    End Sub

    Private Sub TurnLeft()
        Idle()
        myBrick.MotorB.Run(CSByte(-PowerLevel / 2), 0)
        myBrick.MotorC.Run(CSByte(PowerLevel / 2), 0)

    End Sub
    Private Sub TurnRight()
        Idle()
        myBrick.MotorB.Run(CSByte(PowerLevel / 2), 0)
        myBrick.MotorC.Run(CSByte(-PowerLevel / 2), 0)
    End Sub
#End Region

#Region "Idle and Disconnect"
    Private Sub btnDisconnect_Click(sender As System.Object, e As System.EventArgs) Handles btnDisconnect.Click
        Disconnect()
    End Sub

    Private Sub Idle()
        If Not myBrick Is Nothing AndAlso myBrick.IsConnected Then
            motorPair.Idle()
            motorPair.ResetMotorPosition(True)
        End If
    End Sub
    Private Sub btnIdle_Click(sender As System.Object, e As System.EventArgs) Handles btnIdle.Click
        Idle()
    End Sub

    Private Sub Disconnect()
        '---stop motors before quitting
        Idle()
        If Not myBrick Is Nothing AndAlso myBrick.IsConnected Then
            myBrick.Disconnect()
            lblStatus.Text = DisconnectedMsg
        End If
    End Sub

    Private Sub btnExit_Click(sender As System.Object, e As System.EventArgs) Handles btnExit.Click
        Disconnect()
        Me.Close()

    End Sub

    Private Sub frmMain_FormClosing(sender As Object, e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        btnDisconnect.PerformClick()
    End Sub
#End Region

End Class
